// ------------------------------------------------------------------------------
// DPF.Android.JListView Component
//
// Dadeh Pardazane Faragir ( DPF ) Co.
//
// Web: http://www.dpfaragir.com
//
// Developed By: Babak Yaghoobi
//
// Email #1: yaghoobi@dpfaragir.com
// Email #2: b_yaghobi@yahoo.com
//
// ------------------------------------------------------------------------------
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// ------------------------------------------------------------------------------
unit DPF.Android.JListView;

interface

{$I DPF.Android.Defs.inc}

uses
  System.SysUtils,
  System.Classes,
  System.Types,
  System.UITypes,
  System.Variants,
  System.Math,
  System.Generics.Collections,

  System.TypInfo,
  DPF.Android.BaseControl,
{$IFDEF ANDROID}
  DPF.Android.Widget,
  DPF.Android.R,
  Androidapi.JNI.Widget,
  Androidapi.Jni,
  Androidapi.JNIBridge,
  Androidapi.JNI.JavaTypes,
  Androidapi.JNI.GraphicsContentViewText,
  FMX.Helpers.Android,
{$ELSE}
  DPF.Android.DesignTime,
{$ENDIF}
  FMX.Forms,
{$IFDEF DELPHIXE5} FMX.Graphics, {$ENDIF}FMX.Types;

type

  TDPFJListView = class;

{$IFDEF ANDROID}

  TDPFListViewOnItemClickListener = class( TJavaLocal, JAdapterView_OnItemClickListener )
  private
    [weak]
    FDPFJListView: TDPFJListView;
  public
    constructor create( ADPFJListView: TDPFJListView );
    procedure onItemClick( parent: JAdapterView; view: JView; position: Integer; id: Int64 ); cdecl;
  end;

  TDPFListViewOnItemSelectedListener = class( TJavaLocal, JAdapterView_OnItemSelectedListener )
  private
    [weak]
    FDPFJListView: TDPFJListView;
  public
    constructor create( ADPFJListView: TDPFJListView );

    procedure onItemSelected( parent: JAdapterView; view: JView; position: Integer; id: Int64 ); cdecl;
    procedure onNothingSelected( parent: JAdapterView ); cdecl;
  end;

  TDPFListViewOnScrollListener = class( TJavaLocal, JAbsListView_OnScrollListener )
  private
    [weak]
    FDPFJListView: TDPFJListView;
  public
    constructor create( ADPFJListView: TDPFJListView );

    procedure onScroll( view: JAbsListView; firstVisibleItem: Integer; visibleItemCount: Integer; totalItemCount: Integer ); cdecl;
    procedure onScrollStateChanged( view: JAbsListView; scrollState: Integer ); cdecl;
  end;

{$ENDIF}

  TDPFListViewOnItemClick       = procedure( Sender: TObject; const Position: Integer; const ID: Int64 ) of object;
  TDPFListViewOnItemSelected    = procedure( Sender: TObject; const Position: Integer; const ID: Int64 ) of object;
  TDPFListViewOnNothingSelected = procedure( Sender: TObject ) of object;

  [ComponentPlatformsAttribute( PidWin32 or PidWin64 or pidLinux32 or pidAndroid or PidOSX32 or PidiOSDevice or PidiOSSimulator )]
  TDPFJListView = class( TDPFANDBaseControl )
  private
    FOnItemClick      : TDPFListViewOnItemClick;
    FOnItemSelected   : TDPFListViewOnItemSelected;
    FOnNothingSelected: TDPFListViewOnNothingSelected;
  protected
{$IFDEF ANDROID}
    FJListView                        : JListView;
    FDPFListViewOnItemClickListener   : TDPFListViewOnItemClickListener;
    FDPFListViewOnItemSelectedListener: TDPFListViewOnItemSelectedListener;
    FDPFListViewOnScrollListener      : TDPFListViewOnScrollListener;
    FSelectedLists                    : TList<Integer>;
    FLastItemSelected                 : Integer;
{$ENDIF}
    procedure Resize; override;
    procedure Move; override;
{$IFNDEF ANDROID}
    procedure Paint; override;
{$ENDIF}
  public
{$IFDEF ANDROID}
    // property GetJListView: JListView read FJListView;
    procedure Loaded; override;
{$ENDIF}
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;

  published

    property OnItemClick      : TDPFListViewOnItemClick read FOnItemClick write FOnItemClick;
    property OnItemSelected   : TDPFListViewOnItemSelected read FOnItemSelected write FOnItemSelected;
    property OnNothingSelected: TDPFListViewOnNothingSelected read FOnNothingSelected write FOnNothingSelected;

    property Clickable;
    property Focusable;
    property FocusableInTouchMode;
    property BackgroundColor1;
    property BackgroundColor2;
    property BackgroundColor3;
    property BackgroundImage;
    property BorderWidth;
    property BorderColor;
    property BorderCornerRadius;
    property GradientOrientation;
    property Alpha;
    property Align;
    property Position;
    property Width;
    property Height;
    property Visible;
    property OnClick;
  end;

implementation

// ------------------------------------------------------------------------------
{ TDPFJListView }
constructor TDPFJListView.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );
  ControlCaption   := 'ListView';
  BackgroundColor1 := TAlphaColors.Lightsteelblue;
  Visible          := True;

{$IFDEF ANDROID}
  CallInUIThreadAndWaitFinishing(
    procedure( )
    begin
      FJListView := TJListView.JavaClass.init( SharedActivity );
      // FJListView.setId( TJR_id.JavaClass.list );

      FDPFListViewOnItemClickListener := TDPFListViewOnItemClickListener.create( self );
      FJListView.setOnItemClickListener( FDPFListViewOnItemClickListener );

      FDPFListViewOnItemSelectedListener := TDPFListViewOnItemSelectedListener.create( self );
      FJListView.setOnItemSelectedListener( FDPFListViewOnItemSelectedListener );

      FDPFListViewOnScrollListener := TDPFListViewOnScrollListener.create( self );
      FJListView.setOnScrollListener( FDPFListViewOnScrollListener );

      // FSelectedLists:= TArray<Integer>;
      FSelectedLists := TList<Integer>.Create;
      FSelectedLists.Sort;
    end );
  JControl := FJListView;
{$ENDIF}
end;

// ------------------------------------------------------------------------------
destructor TDPFJListView.Destroy;
begin
{$IFDEF ANDROID}
  FDPFListViewOnItemClickListener    := nil;
  FDPFListViewOnItemSelectedListener := nil;
  FDPFListViewOnScrollListener       := nil;
{$ENDIF}
  inherited;
end;

// ------------------------------------------------------------------------------
{$IFDEF ANDROID}

procedure TDPFJListView.Loaded;
var
  aR       : JArrayAdapter;
  gdDefault: JGradientDrawable;
  clrBase  : JColorDrawable;

  clrBlue  : JColorDrawable;
  clrgreen : JColorDrawable;
  clrorange: JColorDrawable;

  selector              : JStateListDrawable;
  State1, State2, State3: TJavaArray<Integer>;
begin
  Resize; // Very Important for Embbeded Forms, Frames

  CallInUIThreadAndWaitFinishing(
    procedure( )
    var
      I: Integer;
    begin
      FJListView.setCacheColorHint( TAlphaColors.Null );
      FJListView.setDrawSelectorOnTop( false );
      FJListView.setSmoothScrollbarEnabled( true );

      selector := TJStateListDrawable.JavaClass.init;

      (*
        // clrBlue := SharedActivity.getResources.getDrawable( TAlphaColors.Blue );
        // clrgreen := SharedActivity.getResources.getDrawable( TAlphaColors.Green );
        // clrorange := SharedActivity.getResources.getDrawable( TAlphaColors.Orange );

        clrBlue := TJColorDrawable.JavaClass.init;
        clrBlue.setColor( TAlphaColors.Blue );

        clrGreen := TJColorDrawable.JavaClass.init;
        clrGreen.setColor( TAlphaColors.Green );

        clrOrange := TJColorDrawable.JavaClass.init;
        clrOrange.setColor( TAlphaColors.Orange );

        State1 := TJavaArray<Integer>.Create( 1 );
        State1[0] := 16842919; // android.R.attr.state_pressed
        selector.addState( State1, clrBlue );

        State2 := TJavaArray<Integer>.Create( 1 );
        State2[0] := 16842912; // -android.R.attr.state_checked
        selector.addState( State2, clrgreen );

        State3 := TJavaArray<Integer>.Create(1);
        State3[0] := 0 ;
        selector.addState( State3, clrorange );

        FJListView.setBackgroundDrawable( selector );
      *)

      clrBase := TJColorDrawable.JavaClass.init;
      clrBase.setColor( TAlphaColors.Red );

      gdDefault := TJGradientDrawable.JavaClass.init;
      gdDefault.setColor( TAlphaColors.Blue );
      gdDefault.setCornerRadius( 0 );
      // gdDefault.setStroke( 1, TAlphaColors.Blue );
      // ListView.setSelector( clrBase { gdDefault } );
      FJListView.setSelector( selector );

      (*
        aR := TJArrayAdapter.JavaClass.init( SharedActivity, TJR_layout.JavaClass.simple_list_item_1 );
        for I := 0 to 60 do
        aR.add( StringToJString( 'Row ' + i.toString ) );

        FJListView.setAdapter( JListAdapter( aR ) );
      *)

    end );
  addSubview( Self, ParentControl );
  // ----------------------------
  // Important
  inherited;
end;
{$ENDIF}

// ------------------------------------------------------------------------------
procedure TDPFJListView.Resize;
begin
{$IFDEF ANDROID}
{$ENDIF}
  inherited;
end;

// ------------------------------------------------------------------------------
procedure TDPFJListView.Move;
begin
  Resize;
end;

{$IFNDEF ANDROID}

// ------------------------------------------------------------------------------
procedure TDPFJListView.Paint;
var
  C: TAlphaColor;
begin
  C := BackgroundColor1;
  if C = TAlphaColors.Null then
    C := TAlphaColors.White;
  InternalPaint( '', TAlphaColors.Black, TDPFTextAlignment.taCenter, C );
end;
{$ENDIF}
{$IFDEF ANDROID}

// ------------------------------------------------------------------------------
// TDPFListViewOnItemClickListener
// ------------------------------------------------------------------------------
constructor TDPFListViewOnItemClickListener.create( ADPFJListView: TDPFJListView );
begin
  inherited create;
  FDPFJListView := ADPFJListView;
end;

// ------------------------------------------------------------------------------
procedure TDPFListViewOnItemClickListener.onItemClick( parent: JAdapterView; view: JView; position: Integer; id: Int64 ); cdecl;
begin
  FDPFJListView.FSelectedLists.Remove( position );
  FDPFJListView.FSelectedLists.Add( position );
  FDPFJListView.FLastItemSelected := position;
  if Assigned( FDPFJListView.OnItemClick ) then
    FDPFJListView.OnItemClick( FDPFJListView, position, id );
end;

// ------------------------------------------------------------------------------
// TDPFListViewOnItemClickListener
// ------------------------------------------------------------------------------
constructor TDPFListViewOnItemSelectedListener.create( ADPFJListView: TDPFJListView );
begin
  inherited create;
  FDPFJListView := ADPFJListView;
end;

// ------------------------------------------------------------------------------
procedure TDPFListViewOnItemSelectedListener.onItemSelected( parent: JAdapterView; view: JView; position: Integer; id: Int64 ); cdecl;
begin
  if Assigned( FDPFJListView.FOnItemSelected ) then
    FDPFJListView.FOnItemSelected( FDPFJListView, position, id );
end;

// ------------------------------------------------------------------------------
procedure TDPFListViewOnItemSelectedListener.onNothingSelected( parent: JAdapterView ); cdecl;
begin
  if Assigned( FDPFJListView.FOnNothingSelected ) then
    FDPFJListView.FOnNothingSelected( FDPFJListView );
end;

// ------------------------------------------------------------------------------
constructor TDPFListViewOnScrollListener.create( ADPFJListView: TDPFJListView );
begin
  inherited create;
  FDPFJListView := ADPFJListView;
end;

// ------------------------------------------------------------------------------
procedure TDPFListViewOnScrollListener.onScroll( view: JAbsListView; firstVisibleItem: Integer; visibleItemCount: Integer; totalItemCount: Integer ); cdecl;
begin

end;

// ------------------------------------------------------------------------------
// scrollState:
// SCROLL_STATE_IDLE = 0
// SCROLL_STATE_TOUCH_SCROLL = 1
// SCROLL_STATE_FLING = 2
procedure TDPFListViewOnScrollListener.onScrollStateChanged( view: JAbsListView; scrollState: Integer ); cdecl;
begin
end;

{$ENDIF}
// ------------------------------------------------------------------------------

end.
